(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.logger = factory());
}(this, (function () { 'use strict';

var assign = Object.assign;


var humanize = function humanize(ms) {
  if (ms >= 1000) {
    return ms / 1000 + 's';
  }

  return ms + 'ms';
};

var stringToRGB = function stringToRGB(string) {
  var code = string.split('').reduce(function (hash, _, i) {
    return '' + (string.charCodeAt(i) + (hash << 5) - hash);
  }, 0);

  var hex = (code & 0x00FFFFFF).toString(16);
  return '#' + ('00000'.slice(0, 6 - hex.length) + hex);
};

var cloneTree = function cloneTree(tree) {
  return assign({}, tree, {
    attributes: assign({}, tree.attributes),
    childNodes: tree.childNodes.map(function (vTree) {
      return cloneTree(vTree);
    })
  });
};

/**
 * Re-usable log function. Used for during render and after render.
 *
 * @param message - Prefix for the console output.
 * @param method - Which console method to call
 * @param color - Which color styles to use
 * @param date - A date object to render
 * @param transaction - Contains: domNode, oldTree, newTree, patches, promises
 * @param options - Middleware options
 */
var log = function log(message, method, color, date, transaction, completed) {
  var domNode = transaction.domNode,
      oldTree = transaction.oldTree,
      newTree = transaction.newTree,
      patches = transaction.patches,
      promises = transaction.promises,
      options = transaction.options,
      markup = transaction.markup,
      state = transaction.state;

  // Shadow DOM rendering...

  if (domNode.host) {
    var ctorName = domNode.host.constructor.name;


    console[method]('%c' + ctorName + ' render ' + (completed ? 'ended' : 'started'), 'color: ' + stringToRGB(ctorName) + (completed ? '; opacity: 0.5' : ''), completed ? completed : '');
  } else {
    console[method](message, color, completed ? completed : '');
  }

  if (!completed && domNode) {
    console.log('%cdomNode %O', 'font-weight: bold; color: #333', domNode);
  }

  if (!completed && markup) {
    console.log('%cmarkup %O', 'font-weight: bold; color: #333', markup);
  }

  if (!completed && options) {
    console.log('%coptions', 'font-weight: bold; color: #333', options);
  }

  if (oldTree || newTree) {
    console.log('%coldTree %O newTree %O', 'font-weight: bold; color: #333', transaction._cloneOldTree, cloneTree(newTree));
  }

  if (patches) {
    console.log('%cpatches %O', 'font-weight: bold; color: #333', patches);
  }

  // Don't clutter the output if there aren't any promises.
  if (promises && promises.length) {
    console.log('%ctransition promises %O', 'font-weight: bold; color: #333', promises);
  }
};

var index = (function (opts) {
  return function loggerTask(transaction) {
    var start = new Date();

    log('%cdiffHTML...render transaction started', 'group', 'color: #FF0066', start, transaction);

    var oldTree = transaction.state.oldTree;


    if (transaction.state.isRendering) {
      console.groupEnd();

      log('%cdiffHTML...render transaction aborted  ', 'group', 'color: #FF78B2', new Date(), transaction);

      console.groupEnd();
    }

    transaction._cloneOldTree = oldTree && cloneTree(oldTree);

    /**
     * Transaction is effectively done, but we need to listen for it to actually
     * be finished.
     */
    return function () {
      // Transaction has fully completed.
      transaction.onceEnded(function () {
        console.groupEnd();

        log('%cdiffHTML...render transaction ended  ', 'group', 'color: #FF78B2', new Date(), transaction, ' >> Completed in: ' + humanize(Date.now() - start));

        console.groupEnd();
      });
    };
  };
});

return index;

})));
